#program pro fitovani prechodoveho jevu s podkritickym tlumenim v pythonu 3 a vyse
# je potreba stahnout balicky matplotlib, lmfit , viz nize
# je potreba aktualizovat vstupni parametry viz zvyrazneny text o odstavec nize.

import matplotlib.pyplot as plt #nacteni knihovny pro kresleni, instalace pomocí příkazu: pip install matplotlib
import numpy as np  #nacteni numericke knihovny numpy, instalace pomocí příkazu: pip install numpy
from lmfit import Minimizer, Parameters, fit_report   #nacteni knihovny pro fitovani LMfit, instalace pomocí příkazu: pip install numpy
from uncertainties import ufloat #balicek na vypocet propagace chyb, instalace pomocí příkazu: pip install uncertainties


#########HODNOTY V RADCICH NIZE JE POTREBA AKTUALIZOVAT
#nacteni dat ze souboru do matice data, je treba zadat radky zacatku a konce intervalu, co chci fitovat
data=np.loadtxt('scope_0.csv', delimiter=',',skiprows=3)   # je potreba aktualizovat nazev souboru
RadekZacatek=794  #cislo radku zacatku prechodoveho jevu, pouzije se take pro normalizaci na cas t=0.
RadekKonec=1453    #cislo radku konce prechodoveho jevu
#startovaci hodnoty modelu (v nelinearnim fitu je potreba byt relativne blizko spravnym hodnotam)
R=50  #odpor v Ohm
L=113.3E-3   # indukce v H -  toto by melo byt co nejpresnejsi, prtoze se to pouziva nakonec na vypocet R
C=217.6E-9  #kapacita v F
Ui=4   # startovaci napeti
Uf=-4	# konecne napeti
#########HODNOTY V RADCICH VYSE JE POTREBA AKTUALIZOVAT

t0=data[RadekZacatek,0]  # precte cas zacatku prechodoveho jevu
tdata=data[RadekZacatek:RadekKonec,0]-t0  #nacteni prvniho sloupce -  cas do promenne tdata, odecte se zacatek aby cas zacatku byl nula.
Udata=data[RadekZacatek:RadekKonec,2]  #nacteni tretiho sloupce  -  napeti, do promenne Udata

#vypocet startovacich velicin pro fit (dulezite pro nelinearni fitovani)
o_rez=1/(L*C)**0.5   #rezonancni kruhova freveknce, vztah (6.25)
alpha=R/(2*L)    #koeficient exponencialniho poklesu, vztah (6.24)
omegad=o_rez*np.sqrt(1-(alpha/o_rez)**2)   # tlumena rezonancni kruhova frekvence, , vztah (6.42)
UC5=(Ui-Uf)*np.sqrt(1+(alpha/omegad)**2)   # vypocet startovaci hodnoty amplitudy, vztah (6.46)
phiT=-np.arctan(alpha/omegad)    # vypocet startovaci hodnoty faze, vztah (6.46)

ParStart = Parameters()   # definice objektu  startovacich parametru pro fit ParStart, z balicku lmfit. Je mozno zafixovat vary=False
ParStart.add('alpha', value=alpha,vary=True)  # definice jednotlivych parametru s hodnotama rozumne blizkyma finalnim
ParStart.add('UC5', value=UC5,vary=True)
ParStart.add('phi', value=phiT,vary=True)
ParStart.add('Uf', value=Uf,vary=True)
ParStart.add('omegad', value=omegad,vary=True)
print("Startovaci hodnoty parametru:", ParStart)   # tisk startovacich parametru na obrazovku

def UPodKrit(p,t):  #funkce prechodoveho jevu pro podkriticke tlumeni. V objektu p jsou parametry,  v promenne t je cas
	return p["UC5"]*np.exp(-p["alpha"]*t)*np.cos(p["omegad"]*t+p["phi"])  + p["Uf"]  #vztah (6.41)

def residual(p, t, Udata):     #definice residualu: rozdilu fitovane funkce a dat, v objektu p jsou parametry
	return UPodKrit(p,t) - Udata

AmplOrig =  UPodKrit(ParStart,tdata)  # vypocti startovaci spektrum 
minner = Minimizer(residual, ParStart, fcn_args=(tdata,Udata))  #fitovani
result = minner.minimize( )  # predani vysledku do objektu result
print()
print(fit_report(result))  # vypsani vysledku fitu
FileFit = open('Statistika.dat', 'w+')  #ulozeni statistiky do souboru
print(fit_report(result),file=FileFit)
FileFit.close() 

ParsFit= result.params   # nafitovane hodnoty parametru preda do veliciny ParFit
UFit =  UPodKrit(ParsFit,tdata)   # vypocti nafitovanou funkci 
# ulozeni dat a nafitovaneho spektra do souboru
VystupDataFit = open('VystupDataFit.dat', 'w+')   
print ("t[s]", "\t", "Udata",  "\t", "Ufit",file=VystupDataFit)
for i in range(len(tdata)):
    print (tdata[i],"\t", Udata[i],"\t",UFit[i],  file=VystupDataFit)
VystupDataFit.close()

print()
#vytvoreni promennych ufloat z knihovny uncertainties pro vypocet nejen hodnot, ale i chyb. Prvni parametr je hodnota, druhy chyba
alphaFit=ufloat(ParsFit['alpha'], ParsFit['alpha'].stderr)  
omegadFit=ufloat(ParsFit['omegad'], ParsFit['omegad'].stderr)
Rvysl=2*alphaFit*L
omega0=(omegadFit**2+alphaFit**2)**0.5
f0=omega0/(2*np.pi)
print("Rvysl=",Rvysl, "Ohm,  omega0=",omega0, "s-1,  f0=",f0, "Hz")


#vykresleni dat
plt.plot(tdata,Udata,'b-',label='Data')
plt.plot(tdata,UFit,'r--',label='Fit')
#plt.plot(tdata,AmplOrig,'g-',label='Start model')
plt.legend(loc="upper right")
plt.xlabel('t [s]')
plt.ylabel('U [V]')
plt.savefig('PrechodJevPodKrit.pdf')
plt.show()



